@@ -24,7 +24,30 @@
 using namespace UCOMMON_NAMESPACE;
 using namespace SIPWITCH_NAMESPACE;
 
-extern "C" int main(int argc, char **argv)
+#ifdef  _MSWINDOWS_
+#include <io.h>
+static char *getpass(const char *prompt)
+{
+    static char buf[128];
+    size_t i;
+
+    fputs(prompt, stderr);
+    fflush(stderr);
+    for (i = 0; i < sizeof(buf) - 1; i++) {
+        buf[i] = fgetc(stdin);
+        if (buf[i] == '\r' || buf[i] == '\n')
+            break;
+        fputs("*", stderr);
+        fflush(stderr);
+    }
+    buf[i] = 0;
+    fputs("\n", stderr);
+    fflush(stderr);
+    return buf;
+}
+#endif
+
+PROGRAM_MAIN(argc, argv)
 {
     char *realm = NULL, *secret, *verify;
     const char *mode = "md5";
@@ -52,27 +75,22 @@ extern "C" int main(int argc, char **argv)
     }
 
 #ifdef  HAVE_PWD_H
-    if(user && getuid() != 0) {
-        fprintf(stderr, "*** sippasswd: only root can change other user's digests\n");
-        exit(1);
-    }
+    if(user && getuid() != 0)
+        shell::errexit(3, "*** sippasswd: only root can change other user's digests\n");
+
     if(!user) {
         struct passwd *pwd = getpwuid(getuid());
-        if(!pwd) {
-            fprintf(stderr, "*** sippasswd: user id cannot be determined\n");
-            exit(3);
-        }
+        if(!pwd)
+            shell::errexit(3, "*** sippasswd: user id cannot be determined\n");
+
         user = strdup(pwd->pw_name);
     }
-    if(geteuid() != 0) {
-        fprintf(stderr, "*** sippasswd: root privilege required\n");
-        exit(3);
-    }
+    if(geteuid() != 0)
+        shell::errexit(3, "*** sippasswd: root privilege required\n");
 #else
-    if(!user) {
-        fprintf(stderr, "*** sippasswd: user id not specified\n");
-        exit(3);
-    }
+    if(!user)
+        shell::errexit(3, "*** sippasswd: user id not specified\n");
+
 #endif
 
     fsys_t fs;
@@ -81,10 +99,9 @@ extern "C" int main(int argc, char **argv)
     if(!is(fs))
         fsys::open(fs, DEFAULT_VARPATH "/lib/sipwitch/uuid", fsys::ACCESS_RDONLY);
 
-    if(!is(fs)) {
-        fprintf(stderr, "*** sippasswd: no realm active\n");
-        exit(4);
-    }
+    if(!is(fs))
+        shell::errexit(4, "*** sippasswd: no realm active\n");
+
     memset(buffer, 0, sizeof(buffer));
     fsys::read(fs, buffer, sizeof(buffer) - 1);
     fsys::close(fs);
@@ -131,10 +148,8 @@ extern "C" int main(int argc, char **argv)
     fsys::close(fs);
 
     fp = fopen(DEFAULT_VARPATH "/lib/sipwitch/digests.db", "r+");
-    if(!fp) {
-        fprintf(stderr, "*** sippasswd: cannot access digest");
-        exit(1);
-    }
+    if(!fp)
+        shell::errexit(1, "*** sippasswd: cannot access digest");
 
     for(;;) {
         fgetpos(fp, &pos);
@@ -169,6 +184,6 @@ extern "C" int main(int argc, char **argv)
     }
 
     printf("digest updated\n");
-    exit(0);
+    PROGRAM_EXIT(0);
 }
 
